@   echo off

    rem @(#) Version: 2009-05-03

    call :read_cfg "DOSBox-CLU.cfg.txt"
    call :read_cfg "DOSBox-CLU.%ComputerName%.cfg.txt"

    if not defined DOSBOX	set DOSBOX=%ProgramFiles%\DOSBox-0.73\dosbox
    if not defined UNPACKER	set UNPACKER=%ProgramFiles%\7-Zip\7z
    if not defined PACKER	set PACKER=%ProgramFiles%\7-Zip\7z
    if not defined UNPACK_DIR	set UNPACK_DIR=Unpacked

    if /i "%1" == "/?"		( shift & goto do_help		)
    if /i "%1" == "/help"	( shift & goto do_help		)
    if /i "%1" == "/unpack"	( shift & goto do_unpack	)
    if /i "%1" == "/run"	( shift & goto do_run		)
    if /i "%1" == "/backup"	( shift & goto do_backup	)
    if /i "%1" == "/restore"	( shift & goto do_restore	)
    if /i "%1" == "/info"	( shift & goto do_info		)
    if /i "%1" == ""		( call :do_unpack & goto do_run )

    echo Unknown option - %1.
    echo.
    call :do_usage
    goto error_exit

:do_usage
    echo Usage: DOSBox-CLU [ /help ^| /unpack ^| /run ^| /backup ^| /restore ^| /info ]
    goto :EOF

:do_help
    echo DOSBox-CLU - a DOSBox Command Line Utility
    echo By MiniMax
    echo.
    echo DOSBOx-CLU uses informationen embedded in a dosbox.conf file to automate the
    echo process of installing and running DOS applications in DOSBox.
    echo.
    echo Here is an example from a dosbox.conf file for a shareware episode of
    echo Death Rally:
    echo.
    echo   # Title:        Death Rally v1.1 - Shareware episode
    echo   # Publisher:    Apogee Software
    echo   # Year:         1996
    echo   # Link:         http://www.3drealms.com/rally/
    echo   #
    echo   # Archives:     1ral11.zip
    echo   # Config-files: C:\RALLY\DR.CFG
    echo   # Save-games:   C:\RALLY\DR.SW?
    echo.
    echo   [autoexec]
    echo   mount C "C-drive"
    echo   mount D "Unpacked" -t cdrom
    echo.
    echo The important parts are the Archives, Config-files, and Save-games entries.
    echo Running DOSBox-CLU from the same folder as this dosbox.conf file will first
    echo create a folder named "C-drive", and next unpack the file "1ral11.zip" into a
    echo folder called "Unpacked".  Finally DOSBox is launched, and the [autoexec]
    echo section is used to mount "C-drive" as the emulated hard-disk, and "Unpacked" as
    echo the emulated CD-ROM.
    echo.
    echo From there it is all the same, old DOSBox procedure: Switch to D:, run the
    echo installer, switch to C:, run the setup program, and run the game.
    echo.
    echo DOSBox-CLU can also be run with a few command line options:
    echo.
    echo   /unpack   Only do the unpacking.  Do not launch DOSBox.
    echo.
    echo   /run      No unpacking, just launch DOSBox.
    echo.
    echo   /backup   Create 2 ^(Zip^) archives with the files mentioned by the Config-
    echo             -files and Save-games entries.  Useful if you want to save disk-
    echo             space by deleting the "C-drive" and the "Unpacked" folders.
    echo.
    echo   /restore  Unpack the 2 archives "Config-files.zip" and "Save-games.zip" into
    echo             their original location.  Useful after doing a ^(re^)install of the
    echo             game.
    echo.
    echo   /info     Lists some of the comments in the dosbox.conf file.
    echo.
    echo If run *without* any options, DOSBOX-CLU will attempt to first unpack the
    echo archive, and then launch DOSBox.
    echo.
    echo This version of DOSBox-CLU will by default try to use DOSBox 0.72 ^(located in
    echo %ProgramFiles%\DOSBox-0.72^), but that can be changed in different ways.
    echo.
    echo One way is to edit the DOSBox-CLU.cmd script!  The other way is to create a
    echo small configuration file called DOSBox-CLU.cfg.txt ^(use Notepad to create/edit
    echo it^).  In there, you can define four variables:
    echo.
    echo   DOSBOX       Path to the DOSBox executable ^(the EXE-file^).
    echo   UNPACKER     Path to the unpacking ^(unzip^) program.
    echo   PACKER       Path to the packing ^(zip^) program.
    echo   UNPACK_DIR   Folder where archive-files will be unpacked.
    echo.
    echo If not redefined, the following values will be used:
    echo.
    echo    DOSBOX=%ProgramFiles%\DOSBox-0.72\dosbox
    echo    UNPACKER=%ProgramFiles%\7-Zip\7z
    echo    PACKER=%ProgramFiles%\7-Zip\7z
    echo    UNPACK_DIR=Unpacked
    echo.
    echo Finally, you can define these variables as normal environment variables.
    goto :EOF

:do_unpack
    call :confdata "Archives"
    if not defined confdata goto :EOF

    if not exist "%UNPACK_DIR%" (
	for %%F in (%confdata%) do call :unpack %%F "%UNPACK_DIR%"
    )
    if not exist "%UNPACK_DIR%" pause
    goto :EOF

:do_run
    if not exist "C-drive" mkdir "C-drive"
    if not exist "capture" mkdir "capture"

    start "DOSBox" "%DOSBOX%" -conf "dosbox.conf" -noconsole %*
    goto :EOF

:do_backup
    call :confdata "Config-files"
    if defined confdata (
	for %%F in (%confdata:C:\=C-drive\%) do (
	    call :pack "%%F" "Config-files.zip"
	)
    )

    call :confdata "Save-games"
    if defined confdata (
	for %%F in (%confdata:C:\=C-drive\%) do (
	    call :pack "%%F" "Save-games.zip"
	)
    )

    goto :EOF

:do_restore
    if exist "Config-files.zip" (
	call :unpack "Config-files.zip" "."
    )

    if exist "Save-games.zip" (
	call :unpack "Save-games.zip" "."
    )

    goto :EOF

:do_info
    if exist "DOSBox-CLU.cfg.txt" (
	call :show_cfg
    ) else if exist "DOSBox-CLU.%ComputerName%.cfg.txt" (
	call :show_cfg
    )

    findstr /B /C:"# Title:"        "dosbox.conf" 2>NUL: || echo # Title:
    findstr /B /C:"# Developer:"    "dosbox.conf" 2>NUL: || echo # Developer:
    findstr /B /C:"# Publisher:"    "dosbox.conf" 2>NUL: || echo # Publisher:
    findstr /B /C:"# Year:"         "dosbox.conf" 2>NUL: || echo # Year:
    findstr /B /C:"# Genre:"        "dosbox.conf" 2>NUL: || echo # Genre:
    findstr /B /C:"# Status:"       "dosbox.conf" 2>NUL: || echo # Status:
    echo #
    findstr /B /C:"# Link:"         "dosbox.conf" 2>NUL: || echo # Link:
    echo #
    findstr /B /C:"# MD5-sum:"      "dosbox.conf" 2>NUL: || echo # MD5-sum:
    echo #
    findstr /B /C:"# Archives:"     "dosbox.conf" 2>NUL: || echo # Archives:
    findstr /B /C:"# Run:"          "dosbox.conf" 2>NUL: || echo # Run:
    findstr /B /C:"# Setup:"        "dosbox.conf" 2>NUL: || echo # Setup:
    findstr /B /C:"# Config-files:" "dosbox.conf" 2>NUL: || echo # Config-files:
    findstr /B /C:"# Save-games:"   "dosbox.conf" 2>NUL: || echo # Save-games:
    echo #
    findstr /B /C:"# DOSBox:"       "dosbox.conf" 2>NUL: || echo # DOSBox:
    goto :EOF

:unpack
    echo Unpacking "%~1" into "%~2".

    "%UNPACKER%" x -y -o"%~2" "%~1" > NUL: || (
	echo.
	echo ERROR: Unable to unpack file "%~1" into "%~2%"
	echo using the command:
	echo.
	echo.  %UNPACKER% x -o"%~2" "%~1"
	echo.
	echo Please check that you have defined UNPACKER correctly in DOSBox-CLU
	echo Alternatively, you can manually unpack "%~1" into a folder called
	echo "%~2" and then re-run this script.
	echo.
	pause
    )
    goto :EOF

:pack
    echo Packing "%~1" into "%~2"

    "%PACKER%" a -tzip "%~2" "%~1" > NUL: || (
	echo.
	echo ERROR: Unable to pack the files "%~1"
	echo into "%~2" using the command:
	echo.
	echo.  "%PACKER%" a -tzip "%~2" "%~1"
	echo.
	echo Please check that you have defined PACKER correctly in DOSBox-CLU.
	echo Alternatively, you can manually pack the files into an Zip-file
	echo called "%~2".
	echo.
	pause
    )
    goto :EOF

:read_cfg
rem +====================================================================
rem | Read in DOSBox-CLU configuration values.
rem |
rem | The configuration file is a simple text file, where lines starting
rem | with a # is treated as a comment. Everything else should be simple
rem | assignments, e.g.
rem |
rem | UNPACK_DIR=C-drive
rem |
rem | The following variables may be defined:
rem |
rem |   DOSBOX		Path to the DOSBox executable (the EXE-file).
rem |   UNPACKER	Path to the unpacking (unzip) program.
rem |   PACKER		Path to the packing (zip) program.
rem |   UNPACK_DIR	Folder where archive-files will be unpacked.
rem |
rem | Anything else will (hopefully) be silently ignored!
rem +====================================================================

    if exist "%~1" (
	for /F "usebackq eol=# tokens=1,* delims==" %%V in ("%~1") do (
	    if /i "%%V" == "DOSBOX"	set %%V=%%W
	    if /i "%%V" == "UNPACKER"	set %%V=%%W
	    if /i "%%V" == "PACKER"	set %%V=%%W
	    if /i "%%V" == "UNPACK_DIR"	set %%V=%%W
	)
    )
    goto :EOF

:show_cfg
rem +====================================================================
rem | Show DOSBox-CLU configuration values.
rem +====================================================================

    echo # ---
    echo # DOSBOX	= %DOSBOX%
    echo # UNPACKER	= %UNPACKER%
    echo # PACKER	= %PACKER%
    echo # UNPACK_DIR	= %UNPACK_DIR%
    echo # ---
    goto :EOF

:confdata
rem +====================================================================
rem | Get values from dosbox.conf.
rem +====================================================================

    set confdata=
    if not exist "dosbox.conf" goto error_exit

    for /F "tokens=3*" %%D in (
	'findstr /B /C:"# %~1:" dosbox.conf'
    ) do (
	set confdata=%%D
	if not "%%E" == "" set confdata=%%D %%E
    )
    goto :EOF

:error_exit
rem +====================================================================
rem | error_exit - force errorlevel = 1.
rem +====================================================================

    verify 2>NUL: other
    goto :EOF

:no_error
rem +====================================================================
rem | no_error - force errorlevel = 0.
rem +====================================================================

    time >NUL: /t
    goto :EOF
