#!/bin/sh
#
# @(#) Version: 2009-05-03

main() # [--help | --unpack | --run | --backup | --restore | --info]
{
    read_cfg "DOSBox-CLU.cfg.txt"
    read_cfg "DOSBox-CLU.`hostname`.cfg.txt"

        DOSBOX="${DOSBOX:-dosbox}"
      UNPACKER="${UNPACKER:-unzip}"
        PACKER="${PACKER:-zip}"
    UNPACK_DIR="${UNPACK_DIR:-Unpacked}"

    case "$1" in
    --help  )	shift; do_help		;;
    --unpack  )	shift; do_unpack	;;
    --run     )	shift; do_run		;;
    --backup  )	shift; do_backup	;;
    --restore )	shift; do_restore	;;
    --info    )	shift; do_info		;;
    "" )	do_unpack &&
		do_run "$@"
		;;
    * )
		printf >&2 "Error - unknown option: %s\n" "$@"
		do_usage
		exit 1
		;;
    esac

} # main()

do_usage()
{
    printf "Usage: %s [--help | --unpack | --run | --backup | --restore | --info]\n" \
	"`basename "$0"`"

} # do_usage()

do_help()
{
    cat <<-!
	DOSBox-CLU - a DOSBox Command Line Utility
	By MiniMax

	DOSBOx-CLU uses informationen embedded in a dosbox.conf file to automate the
	process of installing and running DOS applications in DOSBox.

	Here is an example from a dosbox.conf file for a shareware episode of
	Death Rally:

	  # Title:        Death Rally v1.1 - Shareware episode
	  # Publisher:    Apogee Software
	  # Year:         1996
	  # Link:         http://www.3drealms.com/rally/
	  #
	  # Archives:     1ral11.zip
	  # Config-files: C:\RALLY\DR.CFG
	  # Save-games:   C:\RALLY\DR.SW?

	  [autoexec]
	  mount C "C-drive"
	  mount D "Unpacked" -t cdrom

	The important parts are the Archives, Config-files and Save-games entries.
	Running DOSBox-CLU from the same folder as this dosbox.conf file will first
	create a folder named "C-drive", and next unpack the file "1ral11.zip" into a
	folder called "Unpacked".  Finally DOSBox is launched, and the [autoexec]
	section is used to mount "C-drive" as the emulated hard-disk, and "Unpacked" as
	the emulated CD-ROM.

	From there it is all the same, old DOSBox procedure: Switch to D:, run the
	installer, switch to C:, run the setup program, and run the game.

	DOSBox-CLU can also be run with a few command line options:

	  --unpack   Only do the unpacking. Do not launch DOSBox.

	  --run      No unpacking, just launch DOSBox.

	  --backup   Create 2 (Zip) archives with the files mentioned by the
	             Config-files and Save-games entries.  Useful if you want
	             to save disk-space by deleting the "C-drive" and the
	             "Unpacked" folders.

	  --restore  Unpack the 2 archives "Config-files.zip" and
	             "Save-games.zip" into their original location.  Useful
	             after doing a (re)install of the game.

	  --info     Lists some of the comments in the dosbox.conf file.


	This version of DOSBox-CLU will by default try to use the version of DOSBox
	that is listed in your PATH, but that can be changed in different ways.

	One way is to edit the DOSBox-CLU.sh script!  The other way is to create a
	small configuration file called DOSBox-CLU.cfg.txt (use any plain text-editor
	to create/edit it).  In there, you can define four variables:

	  DOSBOX       Path to the DOSBox executable (the EXE-file).
	  UNPACKER     Path to the unpacking (unzip) program.
	  PACKER       Path to the packing (zip) program.
	  UNPACK_DIR   Folder where archive-files will be unpacked.

	If not redefined, the following values will be used:

	  DOSBOX="dosbox"
	  UNPACKER="unzip"
	  PACKER="zip"
	  UNPACK_DIR="Unpacked"

	Finally, you can define these variables as normal environment variables.
!
} # do_help()

do_unpack()
{
    get_confdata "Archives" || return

    if [ ! -d "${UNPACK_DIR}" ]; then
	for F in ${confdata}; do unpack "${F}" "${UNPACK_DIR}"; done
    fi
    [ -d "${UNPACK_DIR}" ] || pause

} # do_unpack()

do_run()
{
    [ -d "capture" ] || mkdir "capture"
    [ -d "C-drive" ] || mkdir "C-drive"

    "${DOSBOX}" -conf "dosbox.conf" -noconsole "$@"

} # do_run()

do_backup()
{
    get_confdata "Config-files" || return
    [ -n "${confdata}" ] && {
	fileglob="`unixglob "${confdata}"`"
	fileglob="`echo "${fileglob}" | sed -e 's|/C:/|./C-drive/|g'`"
	pack "${fileglob}" "Config-files.zip"
    }

    get_confdata "Save-games" || return
    [ -n "${confdata}" ] && {
	fileglob="`unixglob "${confdata}"`"
	fileglob="`echo "${fileglob}" | sed -e 's|/C:/|./C-drive/|g'`"
	pack "${fileglob}" "Save-games.zip"
    }

} # do_backup()

do_restore()
{
    [ -f "Config-files.zip" ] &&
	unpack "Config-files.zip" "."

    [ -f "Save-games.zip" ] &&
	unpack "Save-games.zip" "."

} # do_restore()

do_info()
{
    if [ -f "DOSBox-CLU.cfg.txt" ]; then
	show_cfg
    elif [ -f "DOSBox-CLU.`hostname`.cfg.txt" ]; then
	show_cfg
    fi

    egrep "^# Title:"        "dosbox.conf" 2>/dev/null || printf "# Title:\n"
    egrep "^# Developer:"    "dosbox.conf" 2>/dev/null || printf "# Developer:\n"
    egrep "^# Publisher:"    "dosbox.conf" 2>/dev/null || printf "# Publisher:\n"
    egrep "^# Year:"         "dosbox.conf" 2>/dev/null || printf "# Year:\n"
    egrep "^# Genre:"        "dosbox.conf" 2>/dev/null || printf "# Genre:\n"
    egrep "^# Status:"       "dosbox.conf" 2>/dev/null || printf "# Status:\n"
    printf "#\n"
    egrep "^# Link:"         "dosbox.conf" 2>/dev/null || printf "# Link:\n"
    printf "#\n"
    egrep "^# MD5-sum:"      "dosbox.conf" 2>/dev/null || printf "# MD5-sum:\n"
    printf "#\n"
    egrep "^# Archives:"     "dosbox.conf" 2>/dev/null || printf "# Archives:\n"
    egrep "^# Run:"          "dosbox.conf" 2>/dev/null || printf "# Run:\n"
    egrep "^# Setup:"        "dosbox.conf" 2>/dev/null || printf "# Setup:\n"
    egrep "^# Config-files:" "dosbox.conf" 2>/dev/null || printf "# Config-files:\n"
    egrep "^# Save-games:"   "dosbox.conf" 2>/dev/null || printf "# Save-games:\n"
    printf "#\n"
    egrep "^# DOSBox:"       "dosbox.conf" 2>/dev/null || printf "# DOSBox:\n"

} # do_info()

unpack()
{
    printf "Unpacking '%s' into '%s'.\n" "$1" "$2"

    "${UNPACKER}" -qo "$1" -d "$2" || {
	cat <<-!

	ERROR: Unable to unpack file "$1" using the command
	       ${UNPACKER} -qo "$1" -d "$2"

	Please check that you have defined UNPACKER correctly in
	$0.  Alternatively, you can manually unpack "$1"
	into a directory called "$2" and then re-run this script.

!
	pause
	exit
    }

} # unpack()

pack()
{
    printf "Packing '%s' into '%s'.\n" "$1" "$2"

    "${PACKER}" -q "$2" $1 || {
	cat <<-!

	ERROR: Unable to pack files "$1" into "$2"
	using the command

	       ${PACKER} -q -k "$2" $1"

	Please check that you have defined PACKER correctly in
	$0.  Alternatively, you can manually pack the files
	into a Zip-file called "$2".

!
	pause
	exit
    }

} # pack()

###
# Read in DOSBox-CLU configuration values.
#
# The configuration file is a simple text file, where lines starting
# with a # is treated as a comment. Everything else should be simple
# assignments, e.g.
#
# UNPACK_DIR=C-drive
#
# The following variables may be defined:
#
#   DOSBOX	Path to the DOSBox executable (the EXE-file).
#   UNPACKER	Path to the unpacking (unzip) program.
#   PACKER	Path to the packing (zip) program.
#   UNPACK_DIR	Folder where archive-files will be unpacked.
#
# Anything else will (hopefully) be silently ignored!
#/

read_cfg() # config-file
{
    if [ -f "$1" ]; then
	for V in DOSBOX UNPACKER PACKER UNPACK_DIR; do
	    W="`sed -n "/^[ 	]*$V=/s/.*=//p" "$1"`"
	    [ -n "$W" ] && eval $V=\"$W\"
	done
    fi

} # read_cfg()

###
# Show DOSBox-CLU configuration values.
#/

show_cfg()
{
    printf "# ---\n"
    printf "# DOSBOX     = %s\n" "${DOSBOX}"
    printf "# UNPACKER   = %s\n" "${UNPACKER}"  
    printf "# PACKER     = %s\n" "${PACKER}"  
    printf "# UNPACK_DIR = %s\n" "${UNPACK_DIR}"
    printf "# ---\n"

} # show_cfg()

###
# Get values from dosbox.conf.
#/

get_confdata() # keyword
{
    [ -f "dosbox.conf" ] || return 1
    confdata="`sed -n -e "s/
//" -e "/^# $1:[ 	]*/{ s///; p; q}" "dosbox.conf"`"

} # get_confdata()


###
# Feeble attempt at converting a DOS file glob to a Unix ditto.
#
# C:\path\*.? => /C:/path/*.?
#/

unixglob() # dosglob
{
    printf "%s" "${confdata}" | tr '[a-z]' '[A-Z]' |
    sed -e 's/\([A-Z]:\)\\/\/\1\//' -e 's|\\|/|g'

} # unixglob()

###
# Pause....
#/

pause() {
    printf 'Press ENTER to continue... '; read junk

} # pause()

main "$@"
